/**********************************************************************/
/*                                                                    */
/* Copyright (c) 2001 by Sun Microsystems, Inc.                       */
/* All rights reserved.                                               */
/*                                                                    */
/**********************************************************************/

/*
**
** This sample program demonstrates the use of the ECI to perform a Distributed
** Program Link (DPL) to a remote MTP system to determine the date and time.
** The MTP system used is the first system defined in the Sun MTP
** Client configuration file KIXCLI.INI.
**
** When run, this sample program performs two DPLs, the first to determine the
** time on the MTP system, and the second to determine the date. The first
** DPL leaves the logical unit of work in flight. The second DPL runs in this
** same logical unit of work, which is then syncpointed at the end of that DPL.
**
** A single COBOL program is required on the MTP system concerned, called
** ECIEX1S, the source of which can be found in ../MTP/ECIEX1S.clt. This
** program should be compiled and installed on that MTP System.
**
** The MTP system must also be configured at accept TCP/IP connections. See
** the documentation for the MTP 'unikixtcp' process for details on how to
** do this.
**
** 
** NOTE: This sample is designed to illustrate synchronous ECI programming
** techniques, and not as a mission-critical application. It therefore only
** performs minimal checking of return codes from system functions etc.     
*/                                                         

#ifdef _AIX
union wait;
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <cics_eci.h>

typedef enum _dpl
{
    DPL_ASK_TIME,
    DPL_ASK_DATE
} DPL_t;

static CICS_EciSystem_t System;
static cics_slong_t     LuwToken;
static cics_sshort_t 	RetCode;
static unsigned char 	Time[8];
static unsigned char 	Date[8];
static unsigned char 	AbendCode[4];

void DrawData()
{
    /*
    ** This function is responsible for printing the results of the DPLs.
    ** It uses printf() to do this.
    */
    
    printf("System:      %-8.8s\n",System.SystemName);
    if (memcmp(AbendCode,"AEI0",4) == 0)
    {
	printf("Error: PGMIDERR\n");
    }
    else if (memcmp(AbendCode,"AZI4",4) == 0)
    {
	printf("Error: NO MTP\n");
    }
    else
    {
	printf("Date:        %-8.8s\n",Date);
	printf("Time:        %-8.8s\n",Time);
    }

    return;
}


int PerformDPL(DPL_t DPLType)
{
    /*
    ** This function is responsible for performing the actual distributed
    ** program link (DPL) calls to the remote MTP system. It can be called
    ** with a DPL type of DPL_ASK_TIME or DPL_ASK_DATE. If DPL_ASK_TIME is
    ** specified, the DPL request specifies a new logical unit of work (LUW)
    ** which is extended, to allow a subsequent DPL_ASK_DATE reuqest to be run
    ** in that LUW. A DPL_ASK_DATE request is run with the luw token saved from
    ** the previous DPL_ASK_TIME request, and terminates that LUW.
    **/
    ECI_PARMS       EciParms;
    unsigned char   EciEx1Prog[]      = "ECIEX1S "; 
    unsigned char   EciUserID[]       = "        "; 
    unsigned char   EciPassword[]     = "        "; 


    /*
    ** Clean out the ECI parameter block.
    */
    memset(&EciParms,0,sizeof(ECI_PARMS));

    /*
    ** Set the ECI parameter block to our required values
    */

    /*
    ** Use the ECI with ECI_SYNC call type. 
    */ 
    EciParms.eci_call_type		= ECI_SYNC;
    memcpy(EciParms.eci_program_name,  	EciEx1Prog,8);
    memcpy(EciParms.eci_userid,        	EciUserID,8);
    memcpy(EciParms.eci_password,      	EciPassword,8);
    
    if (DPLType == DPL_ASK_TIME)
    {
	memcpy(Time,"ASKTIME:",8);
	EciParms.eci_commarea         	= Time;
    }
    else
    {
	memcpy(Date,"ASKDATE/",8);
	EciParms.eci_commarea		= Date;
    }
    
    EciParms.eci_commarea_length	= 8;
    
    if (DPLType == DPL_ASK_TIME)
    {
	EciParms.eci_extend_mode	= ECI_EXTENDED;
    }
    else
    {
	EciParms.eci_extend_mode	= ECI_NO_EXTEND;
    }
    
    if (DPLType == DPL_ASK_TIME)
    {
	EciParms.eci_luw_token		= ECI_LUW_NEW;
    }
    else
    {
	EciParms.eci_luw_token		= LuwToken;
    }
    
    EciParms.eci_version                = ECI_VERSION_1;
    memcpy(EciParms.eci_system_name, System.SystemName, 8);

    /*
    ** Now perform the actual DPL.
    */
    RetCode = CICS_ExternalCall(&EciParms);
    
    if (RetCode == ECI_NO_ERROR)
    {
	LuwToken = EciParms.eci_luw_token;
	if ( EciParms.eci_system_name[0]!='\0' )
	{
	    memcpy(System.SystemName, EciParms.eci_system_name,8);
	}
	return 1;
    }
    else
    {
	memcpy(AbendCode,EciParms.eci_abend_code,4);
	fprintf(stderr,"CICS_ExternalCall() returned %d\n",RetCode);
	fprintf(stderr,"System:     <%-8.8s>\n",System.SystemName);
	fprintf(stderr,"Abend Code: <%-4.4s>\n",EciParms.eci_abend_code);
    }
	    
    return 0;
}



int main( int ArgC , char ** ArgV )
{
    if ( PerformDPL(DPL_ASK_TIME) )
    {
	if ( PerformDPL(DPL_ASK_DATE) )
	{
	    DrawData();
	}
    }
    return 0;
}
				   

