/**********************************************************************/
/*                                                                    */
/* Copyright (c) 2001 by Sun Microsystems, Inc.                       */
/* All rights reserved.                                               */
/*                                                                    */
/**********************************************************************/

/*
**
** This sample program demonstrates the use of the EPI to invoke a legacy 3270
** application running on MTP as transaction 'EPX1'.
**
** When invoked, the 'EPX1' transaction determines the date and time, and
** displays a map showing this information. The user can then press PF3 to exit,
** or any other AID key to refresh the date and time display.
**
** The 'EPX1' transaction consists of one MTP COBOL program called EPIEX1S
** and a mapset called EPIEX1M containing two bms maps. The source of the COBOL
** program and the BMS mapset can be found in .\MTP\EPIEX1S.clt and
** .\MTP\EPIEX1M.bms respectively.
**
** Appropriate PPT entries should be made for the program/mapset once they
** have been assembled/translated/compiled. A PCT entry for transaction 'EPX1'
** should also be defined.  
**
**
** This sample program uses the EPI to auto-install a terminal, invoke the EPX1
** transaction, parse the output, and display the date and time. The date and
** time are then repeatedly updated by pressing if the 'ENTER' AID until the
** 'stop' pulldown is used. Once this happens, the this application simulates
** the pressing of PF3 on the terminal concerned, and once the 'end' map is
** displayed, the terminal is logged out.  
** 
** NOTE: This sample is designed to illustrate various EPI programming
** techniques, and not as a mission-critical application. It therefore only
** performs minimal checking of return codes from system functions etc.     
*/                                                         

#ifdef _AIX
union wait;
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <cics_epi.h>

/*
** The following defines define the datastreams to be sent when a specific
** key is required.
*/
#define DATASTREAM_ENTER	(cics_ubyte_t *)"'  "
#define DATASTREAM_PF3		(cics_ubyte_t *)"3  "

static cics_ushort_t    	NumSystems = 0;
static cics_sshort_t 		RetCode;
static unsigned char 		Time[8];
static unsigned char 		Date[8];
static cics_ushort_t 		TermIndex;
static CICS_EpiDetails_t	TermDetails;
static CICS_EpiEventData_t	Event;
static cics_char_t		NextTran[4];


static void EvaluateData( char *Data , int Size )
{
    /*
    ** MAP1 is the Date map
    */
    if ( memcmp(Data+0x13,"MAP1",4) == 0 )
    {
	printf("Date         %8.8s\n",Data+0x82);
    }
    /*
    ** MAP2 is the Time map
    */
    else if ( memcmp(Data+0x13,"MAP2",4) == 0 )
    {
	printf("Time         %8.8s\n",Data+0x82);
    }
    /*
    ** MAP3 is the Ended map
    */
    else if ( memcmp(Data+0x13,"MAP3",4) == 0 )
    {
    }
    else if ( memcmp(Data+0x0F,"MAP1",4) == 0 )
    {
	printf("Date         %8.8s\n",Data+0x7E);
    }
    /*
    ** MAP2 is the Time map
    */
    else if ( memcmp(Data+0x0F,"MAP2",4) == 0 )
    {
	printf("Time         %8.8s\n",Data+0x7E);
    }
    /*
    ** MAP3 is the Ended map
    */
    else if ( memcmp(Data+0x0F,"MAP3",4) == 0 )
    {
    }
    else
    {
	printf("Send data is <%s>\n",Data);
	printf("Send data is <%s>\n",Data+0x13);
    }
}

static cics_ushort_t WaitForConverse( cics_ushort_t TermIndex )
{
    cics_ushort_t	epirc;
    int			Done=0;
    CICS_EpiEventData_t	Event;
    char		Buffer[7000];

    while ( !Done )
    {
	memset(Buffer,'\0',7000);
	Event.Data = (cics_ubyte_t *)Buffer;
	Event.Size = 7000;
	epirc = CICS_EpiGetEvent( TermIndex,
				  CICS_EPI_WAIT,
				  &Event );
	if ( ( epirc == CICS_EPI_NORMAL ) ||
	     ( epirc == CICS_EPI_ERR_MORE_EVENTS ) )
	{
	    switch ( Event.Event )
	    {
		case CICS_EPI_EVENT_SEND:
		    if ( Event.Size > 0 )
		    {
			EvaluateData( Buffer, Event.Size );
		    }
		    break;
		case CICS_EPI_EVENT_CONVERSE:
		    if ( Event.Size > 0 )
		    {
			EvaluateData( Buffer, Event.Size );
		    }
		    Done = 1;
		    break;
		case CICS_EPI_EVENT_END_TRAN:
		    fprintf(stderr,"END_TRAN Received Unexpectedly\n");
		    Done = 1;
		    epirc = CICS_EPI_ERR_FAILED;
		    break;
		case CICS_EPI_EVENT_END_TERM:
		    fprintf(stderr,"END_TERM Received Unexpectedly\n");
		    Done = 1;
		    epirc = CICS_EPI_ERR_FAILED;
		    break;
		default:
		    fprintf( stderr,
			     "CICS_EpiGetEvent() gave event %d\n",
			     Event.Event);
		    break;
	    }
	}
	else
	{
	    fprintf(stderr,"CICS_EpiGetEvent() failed with rc=%d\n",epirc);
	    Done = 1;
	}
    }
    return epirc;
}

static cics_ushort_t WaitForEndTran( cics_ushort_t TermIndex )
{
    cics_ushort_t	epirc;
    int			Done=0;
    CICS_EpiEventData_t	Event;
    char		Buffer[7000];

    while ( !Done )
    {
	memset(Buffer,'\0',7000);
	Event.Data = (cics_ubyte_t *)Buffer;
	Event.Size = 7000;
	epirc = CICS_EpiGetEvent( TermIndex,
				  CICS_EPI_WAIT,
				  &Event );
	if ( ( epirc == CICS_EPI_NORMAL ) ||
	     ( epirc == CICS_EPI_ERR_MORE_EVENTS ) )
	{
	    switch ( Event.Event )
	    {
		case CICS_EPI_EVENT_SEND:
		    if ( Event.Size > 0 )
		    {
			EvaluateData( Buffer, Event.Size );
		    }
		    break;
		case CICS_EPI_EVENT_END_TRAN:
		    memcpy(NextTran,Event.TransId,4);
		    Done = 1;
		    break;
		case CICS_EPI_EVENT_END_TERM:
		    epirc = CICS_EPI_ERR_FAILED;
		    Done = 1;
		    break;
		default:
		    fprintf( stderr,
			     "CICS_EpiGetEvent() gave event %d\n",
			     Event.Event);
		    break;
	    }
	}
	else
	{
	    fprintf(stderr,"CICS_EpiGetEvent() failed with rc=%d\n",epirc);
	    Done = 1;
	}
    }
    return epirc;
}

static cics_ushort_t WaitForEndTerm( cics_ushort_t TermIndex )
{
    cics_ushort_t	epirc;
    int			Done=0;
    CICS_EpiEventData_t	Event;
    char		Buffer[7000];

    while ( !Done )
    {
	memset(Buffer,'\0',7000);
	Event.Data = (cics_ubyte_t *)Buffer;
	Event.Size = 7000;
	epirc = CICS_EpiGetEvent( TermIndex,
				  CICS_EPI_WAIT,
				  &Event );
	if ( ( epirc == CICS_EPI_NORMAL ) ||
	     ( epirc == CICS_EPI_ERR_MORE_EVENTS ) )
	{
	    switch ( Event.Event )
	    {
		case CICS_EPI_EVENT_END_TERM:
		    Done = 1;
		    break;
		default:
		    fprintf( stderr,
			     "CICS_EpiGetEvent() gave event %d\n",
			     Event.Event);
		    break;
	    }
	}
	else
	{
	    fprintf(stderr,"CICS_EpiGetEvent() failed with rc=%d\n",epirc);
	    Done = 1;
	}
    }
    return epirc;
}

int main( int ArgC, char **ArgV )
{
    cics_ushort_t	epirc;
    CICS_EpiSystem_t	System;
    cics_ushort_t	NumSystems;
    CICS_EpiSysError_t	SysError;
    cics_ubyte_t	DataStream[256];
    cics_ushort_t	DataStreamSize;

    /********************************************************************/
    /* Initialize							*/
    /********************************************************************/
    epirc = CICS_EpiInitialize( CICS_EPI_VERSION_101 );
    if ( epirc != CICS_EPI_NORMAL )
    {
	fprintf(stderr,"CICS_EpiInitialize() failed with rc=%d\n",epirc);
	exit(1);
    }

    /********************************************************************/
    /* List Systems							*/
    /********************************************************************/
    NumSystems = 1;
    epirc = CICS_EpiListSystems( NULL,
				 &NumSystems,
				 &System );
    if ( ( epirc != CICS_EPI_NORMAL ) &&
	 (epirc != CICS_EPI_ERR_MORE_SYSTEMS) )
    {
	fprintf(stderr,"CICS_EpiListSystems() failed with rc=%d\n",epirc);

	epirc = CICS_EpiTerminate();
	if ( epirc != CICS_EPI_NORMAL )
	{
	    fprintf(stderr,"CICS_EpiTerminate() failed with rc=%d\n",epirc);
	}
	exit(1);
    }

    printf("System:      %-8.8s\n",System.SystemName);

    /********************************************************************/
    /* Add Terminal							*/
    /********************************************************************/
    epirc = CICS_EpiAddTerminal( NULL,
				 System.SystemName,
				 NULL,
				 NULL,
				 NULL,
				 &TermDetails,
				 &TermIndex);
    if ( epirc != CICS_EPI_NORMAL )
    {
	fprintf(stderr,"CICS_EpiAddTerminal() failed with rc=%d\n",epirc);
	memset(&SysError,'\0',sizeof(CICS_EpiSysError_t));
	epirc = CICS_EpiGetSysError( CICS_EPI_TERM_INDEX_NONE,
				     &SysError );
	if ( epirc != CICS_EPI_NORMAL )
	{
	    fprintf(stderr,"CICS_EpiGetSysError() failed with rc=%d\n",epirc);
	}
	fprintf(stderr,"CICS_EpiGetSysError():Cause = %ld\n",SysError.Cause);
	fprintf(stderr,"CICS_EpiGetSysError():Value = %ld\n",SysError.Value);
	epirc = CICS_EpiTerminate();
	if ( epirc != CICS_EPI_NORMAL )
	{
	    fprintf(stderr,"CICS_EpiTerminate() failed with rc=%d\n",epirc);
	}
	exit(1);
    }

    /********************************************************************/
    /* Start Tran							*/
    /********************************************************************/
    memcpy(DataStream,DATASTREAM_ENTER,3);		/* DFHENTER + (0,0) cursor address */
    DataStreamSize = 3;
    
    epirc = CICS_EpiStartTran(	TermIndex,
				"EPX1",
				DataStream,
				DataStreamSize );

    if ( epirc != CICS_EPI_NORMAL )
    {
	fprintf(stderr,"CICS_EpiStartTran() failed with rc=%d\n",epirc);
	epirc = CICS_EpiDelTerminal( TermIndex );
	if ( epirc != CICS_EPI_NORMAL )
	{
	    fprintf(stderr,"CICS_EpiDelTerminal() failed with rc=%d\n",epirc);
	}
	else
	{
	    epirc = WaitForEndTerm( TermIndex );
	}
	epirc = CICS_EpiTerminate();
	if ( epirc != CICS_EPI_NORMAL )
	{
	    fprintf(stderr,"CICS_EpiTerminate() failed with rc=%d\n",epirc);
	}
	exit(1);
    }

    /********************************************************************/
    /* Wait for the converse event					*/
    /********************************************************************/
    epirc = WaitForConverse( TermIndex );
    if ( epirc != CICS_EPI_NORMAL )
    {
	epirc = CICS_EpiDelTerminal( TermIndex );
	if ( epirc != CICS_EPI_NORMAL )
	{
	    fprintf(stderr,"CICS_EpiDelTerminal() failed with rc=%d\n",epirc);
	}
	else
	{
	    epirc = WaitForEndTerm( TermIndex );
	}
	epirc = CICS_EpiTerminate();
	if ( epirc != CICS_EPI_NORMAL )
	{
	    fprintf(stderr,"CICS_EpiTerminate() failed with rc=%d\n",epirc);
	}
	exit(1);
    }

    /********************************************************************/
    /* Press Enter							*/
    /********************************************************************/
    epirc = CICS_EpiReply( TermIndex,
			   DATASTREAM_ENTER,
			   3 );
    if ( epirc != CICS_EPI_NORMAL )
    {
	fprintf(stderr,"CICS_EpiReply() failed with rc=%d\n",epirc);
	epirc = CICS_EpiDelTerminal( TermIndex );
	if ( epirc != CICS_EPI_NORMAL )
	{
	    fprintf(stderr,"CICS_EpiDelTerminal() failed with rc=%d\n",epirc);
	}
	else
	{
	    epirc = WaitForEndTerm( TermIndex );
	}
	epirc = CICS_EpiTerminate();
	if ( epirc != CICS_EPI_NORMAL )
	{
	    fprintf(stderr,"CICS_EpiTerminate() failed with rc=%d\n",epirc);
	}
	exit(1);
    }


    /********************************************************************/
    /* Wait for the end of transaction					*/
    /********************************************************************/
    epirc = WaitForEndTran( TermIndex );

    /********************************************************************/
    /* Press PF3							*/
    /********************************************************************/
    epirc = CICS_EpiStartTran( TermIndex,
			       NextTran,
			       DATASTREAM_PF3,
			       3 );
    if ( epirc != CICS_EPI_NORMAL )
    {
	fprintf(stderr,"CICS_EpiStartTran() failed with rc=%d\n",epirc);
	epirc = CICS_EpiDelTerminal( TermIndex );
	if ( epirc != CICS_EPI_NORMAL )
	{
	    fprintf(stderr,"CICS_EpiDelTerminal() failed with rc=%d\n",epirc);
	}
	else
	{
	    epirc = WaitForEndTerm( TermIndex );
	}
	epirc = CICS_EpiTerminate();
	if ( epirc != CICS_EPI_NORMAL )
	{
	    fprintf(stderr,"CICS_EpiTerminate() failed with rc=%d\n",epirc);
	}
	exit(1);
    }

    /********************************************************************/
    /* Wait for the end of transaction					*/
    /********************************************************************/
    epirc = WaitForEndTran( TermIndex );

    /********************************************************************/
    /* Delete the terminal						*/
    /********************************************************************/
    epirc = CICS_EpiDelTerminal( TermIndex );
    if ( epirc != CICS_EPI_NORMAL )
    {
	fprintf(stderr,"CICS_EpiDelTerminal() failed with rc=%d\n",epirc);
    }
    else
    {
	epirc = WaitForEndTerm( TermIndex );
    }

    /********************************************************************/
    /* Terminate							*/
    /********************************************************************/
    epirc = CICS_EpiTerminate();
    if ( epirc != CICS_EPI_NORMAL )
    {
	fprintf(stderr,"CICS_EpiTerminate() failed with rc=%d\n",epirc);
    }
    return 0;
}

