/**********************************************************************/
/*                                                                    */
/* Copyright (c) 2004 by Sun Microsystems, Inc.                       */
/* All rights reserved.                                               */
/*                                                                    */
/**********************************************************************/

import com.sun.emp.cci.eci.ECIInteractionSpec;
import com.sun.emp.cci.eci.ECIManagedConnectionFactory;
import com.sun.emp.cci.eci.ECITransactionAbendException;
import com.sun.emp.cci.eci.GenericRecord;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;

import javax.resource.cci.Connection;
import javax.resource.cci.ConnectionFactory;
import javax.resource.cci.Interaction;
import javax.resource.cci.InteractionSpec;

import javax.resource.ResourceException;

/**
 * An example unmanaged program using the MTP ECI Resource Adapter.
 * <p>This example obtains the data and time in an extended logical
 * unit of work from an MTP system using the MTP ECI Resource Adapter.
 */
public class Sample2 {

    private ConnectionFactory cf;

    public Sample2(String host, int port, String regionName)
    						throws ResourceException {

	//
	// Create the managed connection factory and configure its
	// properties (as supplied as arguiments to this method)
	//
	ECIManagedConnectionFactory emcf = new ECIManagedConnectionFactory();
	emcf.setNameServiceHost(host);
	emcf.setNameServicePort(new Integer(port));
	emcf.setRegionName(regionName);

	//
	// Use the managed connection factory to create a connection factory
	//
	cf = (ConnectionFactory)emcf.createConnectionFactory();
    }

    /**
     * Perform two queries against the MTP system within the same extended
     * logical unit of work.
     * <p>This method uses the <code>ECIEX1S</code> program on the server
     * to obtain information. There are 2 queries, one for the
     * date and one for the time.
     * @throws javax.resource.ResourceException if anything goes wrong.
     */
    private void performQueries() throws ResourceException {

	//
	// Create and configure the interaction spec. This tells the
	// server what MTP program to run - we are going to call the
	// same program twice within the same logical unit of work.
	//
	ECIInteractionSpec eis = new ECIInteractionSpec();

	eis.setInteractionVerb(InteractionSpec.SYNC_SEND_RECEIVE);
	eis.setFunctionName("ECIEX1S");

	Connection c = null;

	try {
	    //
	    // Get a connection from the connection factory.
	    // This will be 'pooled' by the default connection manager
	    //
	    c = cf.getConnection();

	    //
	    // Start a new logical unit of work
	    //
	    c.getLocalTransaction().begin();

	    //
	    // Create an interaction and
	    // execute the first program within it.
	    //
	    Interaction ia = c.createInteraction();

	    //
	    // Create a record to hold the input commarea. Note that this
	    // will also hold the resultant commarea after the program is
	    // actually executed.
	    //
	    GenericRecord gr = new GenericRecord("ASKDATE/".getBytes());

	    //
	    // Invoke the first remote program in the LUW and store the result
	    //
	    ia.execute(eis, gr, gr);
	    String date = new String(gr.getBytes());
	
	    //
	    // Perform the second query in the LUW and store the result
	    //
	    gr = new GenericRecord("ASKTIME:".getBytes());
	    ia.execute(eis, gr, gr);
	    String time = new String(gr.getBytes());

	    //
	    // Lastly, commit the logical unit of work
	    //
	    c.getLocalTransaction().commit();

	    //
	    // Print the results
	    //
	    System.out.println("Date: " + date);
	    System.out.println("Time: " + time);
        } finally {
	    //
	    // Whether we were successful or not, we must close the connection.
	    // This allows connection pooling to occur correctly and it allows
	    // resources to be released.
	    //
            if (c != null) {
                try {
                    c.close();
                } catch (ResourceException re) {
		    //
		    // Dont worry if this fails
		    //
                }
            }
        }
    }

    public static void main(String[] args) {
	//
	// Get hold of the required arguments.
	//
	if (args.length == 0) {
	    try {
		args = getArgsFromStdIn();
	    } catch (IOException e) {
		usage();
		System.exit(1);
	    }
	}
	if (args.length < 3) {
	    usage();
	    System.exit(1);
	}
	String host = args[0];
	int port = 0;
	try {
	    port = Integer.parseInt(args[1]);
	} catch (NumberFormatException e) {
	    usage();
	    System.exit(1);
	}
	String region = args[2];

	//
	// Actually do the sample
	//
	try {
	    Sample2 sample2 = new Sample2(host,port,region);
	    sample2.performQueries();
	} catch (ResourceException e) {
	    analyseResourceException(e);
	}
    }


//--------------------------------------------------------------------------
// Support methods for the main() function
//--------------------------------------------------------------------------
    /**
     * Read a set of arguments from stdin.
     * <p>This is used if no arguments are supplied on the command line
     * @throws java.io.IOException if the stream closes.
     */
    private static String[] getArgsFromStdIn() throws IOException {
	String[] args = new String[3];
	InputStreamReader isr = new InputStreamReader(System.in);
	BufferedReader br = new BufferedReader(isr);
	System.out.println("Name Service Host (e.g. localhost):");
	args[0] = br.readLine();
	System.out.println("Name Service Port (e.g. 7865):");
	args[1] = br.readLine();
	System.out.println("Region Name (e.g. mtp/mtpsys01):");
	args[2] = br.readLine();
	return args;
    }


    /**
     * Display a usage message.
     */
    private static void usage() {
	System.out.println("Usage: java Sample2 host port region");
    }

    /**
     * Provide a display of a caught resource exception.
     * <p>This demonstrates how to obtain an ABEND code from the exception if
     * the MTP transaction abended.
     * @param e The exception to be analysed.
     */
    private static void analyseResourceException(ResourceException e) {
	if (e instanceof ECITransactionAbendException) {
	    ECITransactionAbendException tae = (ECITransactionAbendException)e;
	    System.err.println(
		"Transaction abended with abend code " + tae.getAbendCode());
	} else {
	    String errorCode = e.getErrorCode();
	    System.err.println(e.getMessage() + " - Error code " + errorCode);
	}
    }
}

